@@ -1,5 +1,5 @@
 /* This file is part of the Zebra server.
-   Copyright (C) 1995-2008 Index Data
+   Copyright (C) 1994-2009 Index Data
 
 Zebra is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -258,8 +258,9 @@ void pr_charBSet (BSetHandle *sh, BSet src, void (*f) (int))
         i1 = trav_BSet (sh, src, ++i);
         if (i1 == i)
         {
-            while ((i1=trav_BSet (sh, src, ++i)) == i)
-                ;
+            do
+                ++i;
+            while ((i1=trav_BSet (sh, src, i)) == i);
             if (i != i0+2)
                 f ('-');
             if (i-1 == '-')
@@ -274,6 +275,7 @@ void pr_charBSet (BSetHandle *sh, BSet src, void (*f) (int))
 /*
  * Local variables:
  * c-basic-offset: 4
+ * c-file-style: "Stroustrup"
  * indent-tabs-mode: nil
  * End:
  * vim: shiftwidth=4 tabstop=8 expandtab
