@@ -1,5 +1,5 @@
 /* This file is part of the Zebra server.
-   Copyright (C) 1995-2008 Index Data
+   Copyright (C) 1994-2009 Index Data
 
 Zebra is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -263,7 +263,7 @@ static int cmd_logf( char *args[], WRBUF outbuff)
 	i=2;
     else
 	lev=YLOG_LOG; /* this is in the default set!*/
-    yaz_log( lev, restargs(args,i));
+    yaz_log( lev, "%s", restargs(args,i));
     return 0; /* ok */
 }
  
@@ -865,6 +865,7 @@ int main (int argc, char ** argv)
 /*
  * Local variables:
  * c-basic-offset: 4
+ * c-file-style: "Stroustrup"
  * indent-tabs-mode: nil
  * End:
  * vim: shiftwidth=4 tabstop=8 expandtab
