@@ -1,5 +1,5 @@
 /* This file is part of the Zebra server.
-   Copyright (C) 1995-2008 Index Data
+   Copyright (C) 1994-2009 Index Data
 
 Zebra is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -770,7 +770,11 @@ static void bend_start(struct statserv_options_block *sob)
 	    char pidstr[30];
 	
 	    sprintf(pidstr, "%ld", (long) getpid());
-	    write(fd, pidstr, strlen(pidstr));
+	    if (write(fd, pidstr, strlen(pidstr)) != strlen(pidstr))
+            {
+                yaz_log(YLOG_ERRNO|YLOG_FATAL, "write fail %s", pidfname);
+                exit(1);
+            }
         }
     }
 #endif
@@ -814,6 +818,7 @@ int main(int argc, char **argv)
 /*
  * Local variables:
  * c-basic-offset: 4
+ * c-file-style: "Stroustrup"
  * indent-tabs-mode: nil
  * End:
  * vim: shiftwidth=4 tabstop=8 expandtab
