@@ -1,5 +1,5 @@
 /* This file is part of the Zebra server.
-   Copyright (C) 1995-2008 Index Data
+   Copyright (C) 1994-2009 Index Data
 
 Zebra is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -767,12 +767,12 @@ int zebraExplain_removeDatabase(ZebraExplainInfo zei, void *update_handle)
 		
 		rec = rec_get(zei->records, zad->sysno);
 		(*zei->updateFunc)(zei->updateHandle, rec, 0);
-		rec_free(&rec);
+		rec_del(zei->records, &rec);
 	    }
 	    /* remove database record keys and delete it */
 	    rec = rec_get(zei->records, zdi->sysno);
 	    (*zei->updateFunc)(zei->updateHandle, rec, 0);
-	    rec_free(&rec);
+	    rec_del(zei->records, &rec);
 
 	    /* remove from list */
 	    *zdip = zdi->next;
@@ -1376,14 +1376,19 @@ int zebraExplain_lookup_attr_str(ZebraExplainInfo zei,
 }
 
 int zebraExplain_trav_ord(ZebraExplainInfo zei, void *handle,
-			  int (*f)(void *handle, int ord))
+			  int (*f)(void *handle, int ord,
+                                   const char *index_type,
+                                   const char *string_index,
+                                   zinfo_index_category_t cat))
 {
     struct zebDatabaseInfoB *zdb = zei->curDatabaseInfo;
     if (zdb)
     {
 	struct zebSUInfoB *zsui = zdb->attributeDetails->SUInfo;
 	for ( ;zsui; zsui = zsui->next)
-	    (*f)(handle,  zsui->info.ordinal);
+	    (*f)(handle,  zsui->info.ordinal,
+                 zsui->info.index_type, zsui->info.str,
+                 zsui->info.cat);
     }
     return 0;
 }
@@ -1633,6 +1638,7 @@ void zebraExplain_loadAttsets(data1_handle dh, Res res)
 /*
  * Local variables:
  * c-basic-offset: 4
+ * c-file-style: "Stroustrup"
  * indent-tabs-mode: nil
  * End:
  * vim: shiftwidth=4 tabstop=8 expandtab
