@@ -1,5 +1,5 @@
 /* This file is part of the Zebra server.
-   Copyright (C) 1995-2008 Index Data
+   Copyright (C) 1994-2009 Index Data
 
 Zebra is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -51,6 +51,16 @@ void tst2(void)
     chrmaptab_destroy(tab);
 }
 
+void tst3(void)
+{
+    /* open empty emptycharmap.chrr */
+    chrmaptab tab = chrmaptab_create(get_srcdir() /* tabpath */,
+				     "emptycharmap.chr" /* file */,
+				     0 /* tabroot */ );
+    YAZ_CHECK(!tab);
+    chrmaptab_destroy(tab);
+}
+
 int main(int argc, char **argv)
 {
     YAZ_CHECK_INIT(argc, argv);
@@ -58,6 +68,7 @@ int main(int argc, char **argv)
 
     tst1();
     tst2();
+    tst3();
 
     YAZ_CHECK_TERM;
 }
@@ -65,6 +76,7 @@ int main(int argc, char **argv)
 /*
  * Local variables:
  * c-basic-offset: 4
+ * c-file-style: "Stroustrup"
  * indent-tabs-mode: nil
  * End:
  * vim: shiftwidth=4 tabstop=8 expandtab
