@@ -1,5 +1,5 @@
 /* This file is part of the Zebra server.
-   Copyright (C) 1995-2008 Index Data
+   Copyright (C) 1994-2009 Index Data
 
 Zebra is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -30,30 +30,18 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #include <string.h>
 #include <idzebra/version.h>
 
-void zebra_get_version(char *version_str, char *sys_str)
+void zebra_get_version(char *version_str, char *sha1_str)
 {
     if (version_str)
         strcpy(version_str, ZEBRAVER);
-    if (sys_str)
-    {
-        strcpy(sys_str, "unknown");
-
-#ifdef WIN32
-        strcpy(sys_str, "win32");
-#ifdef _MSC_VER
-        yaz_snprintf(sys_str+strlen(sys_str), 25, "; mscver %lu",
-                 (unsigned long) _MSC_VER);
-#endif
-#endif
-#ifdef HOST_TRIPLET
-        strcpy(sys_str, HOST_TRIPLET);
-#endif
-    }
+    if (sha1_str)
+        strcpy(sha1_str, ZEBRA_VERSION_SHA1);
 }
 
 /*
  * Local variables:
  * c-basic-offset: 4
+ * c-file-style: "Stroustrup"
  * indent-tabs-mode: nil
  * End:
  * vim: shiftwidth=4 tabstop=8 expandtab
